@@ -38,6 +38,7 @@ module Tests.Properties (
 
 import           Prelude hiding (decodeFloat, encodeFloat)
 
+import qualified Data.ByteString as BS
 import qualified Data.ByteString.Lazy as LBS
 import           Data.Word
 import           Data.Int
@@ -47,6 +48,9 @@ import           Data.Function (on)
 import           Data.Proxy
 import           Data.Kind (Type)
 
+import GHC.Exts
+import           Codec.CBOR.ByteArray
+import qualified Codec.CBOR.ByteArray.Sliced as Sliced
 import           Codec.CBOR.Term
 import           Codec.CBOR.Read
 import           Codec.CBOR.Write
@@ -56,10 +60,10 @@ import           Codec.CBOR.Encoding
 import           Test.Tasty (TestTree, testGroup, localOption)
 import           Test.Tasty.QuickCheck (testProperty, QuickCheckMaxSize(..))
 import           Test.QuickCheck
-import           System.Random (Random)
+import           Test.QuickCheck.Gen (Gen (MkGen))
 
 import qualified Tests.Reference.Implementation as Ref
-import           Tests.Reference.Implementation (UInt(..))
+import           Tests.Reference.Implementation (UInt(..), lengthUInt)
 import           Tests.Reference.Generators
 import           Tests.Term
                    ( fromRefTerm, toRefTerm, eqTerm, canonicaliseTerm )
@@ -70,6 +74,15 @@ import           Control.Applicative
 #endif
 
 
+#if MIN_VERSION_bytestring(0,11,1)
+import qualified Data.ByteString.Short as SBS
+#else
+import qualified Data.ByteString.Short.Internal as SBS
+#endif
+
+import qualified Data.Primitive.ByteArray as Prim (ByteArray (..))
+import System.Random.Stateful hiding (genByteString, genShortByteString)
+
 -- | The CBOR implementation and its reference implementation satisfy all the
 -- properties implied in the following commuting diagram.
 --
@@ -984,7 +997,53 @@ instance Token Ref.Term where
     encodeRef   = Ref.encodeTerm
     decodeRef _ = Ref.decodeTerm
 
+--------------------------------------------------------------------------------
+-- Token class instances for ByteArray tokens.
+--
 
+newtype TokByteArray = TokByteArray { unTokByteArray :: [Word8] }
+  deriving (Eq, Show)
+
+instance Arbitrary TokByteArray where
+  arbitrary = TokByteArray <$> arbitrary
+
+instance Token TokByteArray where
+    type Imp TokByteArray = Sliced.SlicedByteArray
+
+    eqImp _ = (==)
+
+    fromRef = Sliced.fromByteString . BS.pack . unTokByteArray
+    toRef _ = TokByteArray . toList
+
+    canonicaliseImp _ = id
+    canonicaliseRef   = id
+
+    encodeImp _ = encodeByteArray
+    decodeImp _ = toSliced <$> decodeByteArray
+
+    encodeRef (TokByteArray bs) = Ref.encodeToken (Ref.MT2_ByteString (lengthUInt bs) bs)
+    decodeRef _ = do Ref.MT2_ByteString _n bs <- Ref.decodeToken
+                     -- TODO? check _n == bs
+                     return (TokByteArray bs)
+
+instance Arbitrary Sliced.SlicedByteArray where
+    -- Taken from cardano-ledger-binary testlib.
+    arbitrary = do
+      NonNegative off <- arbitrary
+      Positive count <- arbitrary
+      NonNegative slack <- arbitrary
+      let len = off + count + slack
+      ba <- genByteArray len
+      pure $ Sliced.SBA ba off count
+
+      where
+        genShortByteString :: Int -> Gen SBS.ShortByteString
+        genShortByteString n = MkGen (\r _n -> runStateGen_ r (uniformShortByteString n))
+
+        genByteArray :: Int -> Gen Prim.ByteArray
+        genByteArray n = do
+                  SBS.SBS ba <- genShortByteString n
+                  pure (Prim.ByteArray ba)
 --------------------------------------------------------------------------------
 -- TestTree API
 
@@ -1011,6 +1070,7 @@ testTree =
     , testProperty "Tag64"   (prop_fromRefToRef (Proxy :: Proxy TokTag64))
     , testProperty "Simple"  (prop_fromRefToRef (Proxy :: Proxy Ref.Simple))
     , testProperty "Term"    (prop_fromRefToRef (Proxy :: Proxy Ref.Term))
+    , testProperty "ByteArray" (prop_fromRefToRef (Proxy :: Proxy TokByteArray))
     ]
 
   , testGroup "from . id . to = canon_imp"
@@ -1033,6 +1093,7 @@ testTree =
     , testProperty "Tag64"   (prop_toRefFromRef (Proxy :: Proxy TokTag64))
     , testProperty "Simple"  (prop_toRefFromRef (Proxy :: Proxy Ref.Simple))
     , testProperty "Term"    (prop_toRefFromRef (Proxy :: Proxy Ref.Term))
+    , testProperty "ByteArray" (prop_toRefFromRef (Proxy :: Proxy TokByteArray))
     ]
 
   , testGroup "dec_ref . enc_ref = id"
@@ -1055,6 +1116,7 @@ testTree =
     , testProperty "Tag64"   (prop_encodeRefdecodeRef (Proxy :: Proxy TokTag64))
     , testProperty "Simple"  (prop_encodeRefdecodeRef (Proxy :: Proxy Ref.Simple))
     , testProperty "Term"    (prop_encodeRefdecodeRef (Proxy :: Proxy Ref.Term))
+    , testProperty "ByteArray" (prop_encodeRefdecodeRef (Proxy :: Proxy TokByteArray))
     ]
 
   , testGroup "dec_imp . enc_imp = canon_imp"
@@ -1077,6 +1139,7 @@ testTree =
     , testProperty "Tag64"   (prop_encodeImpdecodeImp (Proxy :: Proxy TokTag64))
     , testProperty "Simple"  (prop_encodeImpdecodeImp (Proxy :: Proxy Ref.Simple))
     , testProperty "Term"    (prop_encodeImpdecodeImp (Proxy :: Proxy Ref.Term))
+    , testProperty "ByteArray" (prop_encodeImpdecodeImp (Proxy :: Proxy TokByteArray))
     ]
 
   , testGroup "dec_imp . enc_imp = canon_imp (all 2-splits)"
@@ -1100,6 +1163,7 @@ testTree =
     , testProperty "Simple"  (prop_encodeImpdecodeImp_splits2 (Proxy :: Proxy Ref.Simple))
     , localOption (QuickCheckMaxSize 100) $
       testProperty "Term"    (prop_encodeImpdecodeImp_splits2 (Proxy :: Proxy Ref.Term))
+    , testProperty "ByteArray" (prop_encodeImpdecodeImp_splits2 (Proxy :: Proxy TokByteArray))
     ]
 
   , testGroup "dec_imp . enc_imp = canon_imp (all 3-splits)"
@@ -1123,6 +1187,7 @@ testTree =
     , testProperty "Simple"  (prop_encodeImpdecodeImp_splits3 (Proxy :: Proxy Ref.Simple))
     , localOption (QuickCheckMaxSize 25) $
       testProperty "Term"    (prop_encodeImpdecodeImp_splits3 (Proxy :: Proxy Ref.Term))
+    , testProperty "ByteArray" (prop_encodeImpdecodeImp_splits3 (Proxy :: Proxy TokByteArray))
     ]
 
   , testGroup "enc_imp . from = enc_ref . canon_ref"
@@ -1145,6 +1210,7 @@ testTree =
     , testProperty "Tag64"   (prop_encodeRefencodeImp1 (Proxy :: Proxy TokTag64))
     , testProperty "Simple"  (prop_encodeRefencodeImp1 (Proxy :: Proxy Ref.Simple))
     , testProperty "Term"    (prop_encodeRefencodeImp1 (Proxy :: Proxy Ref.Term))
+    , testProperty "ByteArray" (prop_encodeRefencodeImp1 (Proxy :: Proxy TokByteArray))
     ]
 
   , testGroup "enc_ref . to = enc_imp"
@@ -1167,6 +1233,7 @@ testTree =
     , testProperty "Tag64"   (prop_encodeRefencodeImp2 (Proxy :: Proxy TokTag64))
     , testProperty "Simple"  (prop_encodeRefencodeImp2 (Proxy :: Proxy Ref.Simple))
     , testProperty "Term"    (prop_encodeRefencodeImp2 (Proxy :: Proxy Ref.Term))
+    , testProperty "ByteArray" (prop_encodeRefencodeImp2 (Proxy :: Proxy TokByteArray))
     ]
 
   , testGroup "dec_imp . enc_ref = from . dec_ref . enc_ref"
@@ -1189,5 +1256,6 @@ testTree =
     , testProperty "Tag64"   (prop_decodeRefdecodeImp (Proxy :: Proxy TokTag64))
     , testProperty "Simple"  (prop_decodeRefdecodeImp (Proxy :: Proxy Ref.Simple))
     , testProperty "Term"    (prop_decodeRefdecodeImp (Proxy :: Proxy Ref.Term))
+    , testProperty "ByteArray" (prop_decodeRefdecodeImp (Proxy :: Proxy TokByteArray))
     ]
   ]
